
# q-expansion of newform 648.2.d.h, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 648
weight = 2
poly_data = [4, 0, -2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 0, 1], 2], [[-1, 0, 1, 0], 1]]

hecke_ring_character_values = [[487, [1, 0, 0, 0]], [325, [-1, 0, 0, 0]], [569, [1, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0], [0, 0, 0, 0], [0, 0, 1, 1], [2, 2, -1, 0], [0, 0, 1, -2], [0, 0, -3, -1], [-3, 0, 0, 0], [0, 0, 3, 0], [0, -2, 1, 0], [0, 0, 1, -3], [2, -4, 2, 0], [0, 0, 3, -3], [0, 4, -2, 0], [0, 0, -3, -2], [6, -4, 2, 0], [0, 0, -2, 4], [0, 0, 4, 0], [0, 0, 3, -5], [0, 0, 3, -4], [-6, -6, 3, 0], [-1, -4, 2, 0], [-4, 6, -3, 0], [0, 0, 4, -2], [3, 12, -6, 0], [8, -4, 2, 0], [0, 0, -8, -2], [2, -12, 6, 0], [0, 0, -8, -2], [0, 0, -3, 3], [-3, 4, -2, 0], [-4, 2, -1, 0], [0, 0, 1, 4], [-9, -4, 2, 0], [0, 0, 12, 2], [0, 0, -11, 1], [8, 8, -4, 0], [0, 0, 3, 5], [0, 0, 0, 12], [18, -2, 1, 0], [0, 0, -5, 1], [0, 0, 4, -2], [0, 0, -6, 0], [6, 2, -1, 0], [5, 8, -4, 0], [0, 0, -5, -5], [2, -4, 2, 0], [0, 0, -3, 2], [-4, 6, -3, 0], [0, 0, 1, 12], [0, 0, -3, 5], [9, 0, 0, 0], [-18, 4, -2, 0], [-13, 0, 0, 0], [0, 0, 13, -6], [3, 4, -2, 0], [-18, -4, 2, 0], [0, 0, -5, -9], [-16, 2, -1, 0], [0, 0, -12, -8], [-3, -4, 2, 0], [0, 0, 0, 14], [0, 0, 13, -3], [0, 0, 12, -6], [12, -8, 4, 0], [5, 8, -4, 0], [0, 0, -5, 13], [0, 0, 15, 4], [-16, -12, 6, 0], [0, 0, -11, -8], [0, 0, -12, 10], [6, -16, 8, 0], [-12, -18, 9, 0], [-16, -16, 8, 0], [0, 0, 0, 8], [0, 0, 12, 6], [-30, -2, 1, 0], [0, 0, -2, -8], [0, 0, 9, -3], [3, -20, 10, 0], [11, -16, 8, 0], [0, 0, 4, 12], [0, 0, 9, -5], [-6, 12, -6, 0], [-13, -4, 2, 0], [26, -4, 2, 0], [0, 0, -8, -6], [0, 0, 0, 0], [-1, 20, -10, 0], [0, 0, 16, -8], [-4, 8, -4, 0], [0, 0, -8, -8], [30, -10, 5, 0], [2, -12, 6, 0], [0, 0, 7, 10], [0, 0, -9, 14], [0, 24, -12, 0], [0, 0, 4, 16], [-12, 0, 0, 0], [0, 0, -9, -6], [0, 0, -15, -3], [0, 0, 0, -14], [0, 0, 1, 13], [0, 0, -20, -2], [33, -4, 2, 0], [0, 0, 0, -16], [29, 0, 0, 0], [0, 0, -5, -2], [21, -12, 6, 0], [6, -20, 10, 0], [11, 8, -4, 0], [2, 18, -9, 0], [0, 0, -6, 0], [-3, -32, 16, 0], [0, 0, 0, 4], [32, 8, -4, 0], [21, -4, 2, 0], [0, 0, 24, 2], [-30, 6, -3, 0], [0, 0, -14, -8], [0, 0, -11, -20], [0, 0, 9, 5], [-7, -4, 2, 0], [0, 0, -8, 0], [0, 0, 4, 18], [0, 0, 27, -8], [0, 0, 13, 1], [0, 0, -3, 1], [0, -6, 3, 0], [14, 2, -1, 0], [0, 0, 6, -4], [0, 0, 12, -6], [18, 4, -2, 0], [-22, 2, -1, 0], [0, 0, 6, -12], [3, 16, -8, 0], [-7, -28, 14, 0], [0, 0, -5, -3], [0, 0, 3, -10], [0, 0, 13, -5], [9, -12, 6, 0], [0, 0, 0, 12], [0, 0, 19, 3], [14, -4, 2, 0], [0, 0, -5, 6], [0, 0, 6, 12], [-24, 2, -1, 0], [0, 0, -24, -8], [9, 32, -16, 0], [0, 0, -24, 2], [0, 18, -9, 0], [0, 0, -27, -7], [36, 0, 0, 0], [0, 0, 0, -6], [36, 10, -5, 0], [0, 0, -24, -8], [12, -22, 11, 0], [8, -10, 5, 0], [21, 8, -4, 0], [11, 8, -4, 0], [0, 0, 25, 1], [0, 0, -29, 4], [-3, 12, -6, 0], [38, -10, 5, 0], [0, 0, 7, -20], [36, 16, -8, 0], [-12, 8, -4, 0], [-34, 2, -1, 0], [0, 0, -15, 19]]
